/// <reference types="node" />
import { Primitive } from '@sindresorhus/is';
import { Document, HydratedDocument, Model, Schema, Types } from 'mongoose';
import 'reflect-metadata';
import { Class, Constructor } from 'type-fest';
export * from './model';
export * from './schema';
export * from './model-helper';
export * from './middleware';
export { default as validators } from './validator';
export declare const ObjectId: typeof Types.ObjectId;
export type ObjectId = Types.ObjectId;
type ArrayType<T> = T extends Primitive ? Types.Array<T> : Types.Array<Types.Subdocument & T>;
export type DocumentType<T> = HydratedDocument<T>;
export type RichDocumentType<T extends {
    _id?: unknown;
}> = {
    [TKey in keyof T]: T[TKey] extends Array<infer TValue> ? ArrayType<TValue> : T[TKey] extends Buffer ? Types.Buffer : T[TKey] extends Date ? Date : T[TKey] extends Record<string, unknown> ? Types.Subdocument & T[TKey] : T[TKey];
} & Document<T['_id']>;
/** @deprecated use RichModelType<typeof ModelClass> */
export type ModelType<T, THelper = object> = Model<DocumentType<T>, THelper>;
export type RichModelType<T extends Constructor<object>, THelper = object> = Model<InstanceType<T>, THelper> & T;
export type Ref<T extends {
    _id?: unknown;
}> = T['_id'] | T;
export type RefDocument<T extends {
    _id?: unknown;
}> = T['_id'] | DocumentType<T>;
export declare function getSchema<T>(modelClass: Class<T>): Schema;
export declare function getModel<T extends Constructor<object>>(modelClass: T): RichModelType<T>;
export declare function getModelName<T extends Constructor<object>>(modelClass: T): string;
export declare function forNestModule<T extends Constructor<object>>(modelClass: T): {
    name: string;
    schema: Schema;
};
